@@ -1,5 +1,5 @@
 /*
- * $Id: SquidString.h,v 1.16 2008/02/11 23:01:23 amosjeffries Exp $
+ * $Id$
  *
  * DEBUG: section 67    String
  * AUTHOR: Duane Wessels
@@ -40,6 +40,12 @@
 /** todo checks to wrap this include properly */
 #include <ostream>
 
+/* squid string placeholder (for printf) */
+#ifndef SQUIDSTRINGPH
+#define SQUIDSTRINGPH "%.*s"
+#define SQUIDSTRINGPRINT(s) s.psize(),s.rawBuf()
+#endif /* SQUIDSTRINGPH */
+
 
 #define DEBUGSTRINGS 0
 #if DEBUGSTRINGS
@@ -86,6 +92,9 @@ public:
     String (String const &);
     ~String();
 
+    typedef size_t size_type; //storage size intentionally unspecified
+    const static size_type npos = std::string::npos;
+
     String &operator =(char const *);
     String &operator =(String const &);
     bool operator ==(String const &) const;
@@ -95,10 +104,31 @@ public:
      * Retrieve a single character in the string.
      \param pos	Position of character to retrieve.
      */
-    _SQUID_INLINE_ char &operator [](unsigned int pos);
+    _SQUID_INLINE_ char operator [](unsigned int pos) const;
 
-    _SQUID_INLINE_ int size() const;
-    _SQUID_INLINE_ char const * buf() const;
+    _SQUID_INLINE_ size_type size() const;
+    /// variant of size() suited to be used for printf-alikes.
+    /// throws when size() > MAXINT
+    int psize() const;
+
+    /**
+     * \retval true the String has some contents
+     */
+    _SQUID_INLINE_ bool defined() const;
+    /**
+     * \retval true the String does not hold any contents
+     */
+    _SQUID_INLINE_ bool undefined() const;
+    /**
+     * Returns a raw pointer to the underlying backing store. The caller has been
+     * verified not to make any assumptions about null-termination
+     */
+    _SQUID_INLINE_ char const * rawBuf() const;
+    /**
+     * Returns a raw pointer to the underlying backing store.
+     * The caller requires it to be null-terminated.
+     */
+    _SQUID_INLINE_ char const * termedBuf() const;
     void limitInit(const char *str, int len); // TODO: rename to assign()
     void clean();
     void reset(char const *str);
@@ -107,49 +137,45 @@ public:
     void append(char const);
     void append (String const &);
     void absorb(String &old);
-    _SQUID_INLINE_ const char * pos(char const *) const;
-    _SQUID_INLINE_ const char * pos(char const ch) const;
-    _SQUID_INLINE_ const char * rpos(char const ch) const;
+    const char * pos(char const *aString) const;
+    const char * pos(char const ch) const;
+    ///offset from string start of the first occurrence of ch
+    /// returns String::npos if ch is not found
+    size_type find(char const ch) const;
+    size_type find(char const *aString) const;
+    const char * rpos(char const ch) const;
+    size_type rfind(char const ch) const;
     _SQUID_INLINE_ int cmp (char const *) const;
-    _SQUID_INLINE_ int cmp (char const *, size_t count) const;
+    _SQUID_INLINE_ int cmp (char const *, size_type count) const;
     _SQUID_INLINE_ int cmp (String const &) const;
     _SQUID_INLINE_ int caseCmp (char const *) const;
-    _SQUID_INLINE_ int caseCmp (char const *, size_t count) const;
+    _SQUID_INLINE_ int caseCmp (char const *, size_type count) const;
+    _SQUID_INLINE_ int caseCmp (String const &) const;
 
-    /** \deprecated Use assignment to [] position instead.
-     *              ie   str[0] = 'h';
-     */
-    _SQUID_INLINE_ void set(char const *loc, char const ch);
+    String substr(size_type from, size_type to) const;
 
-    /** \deprecated Use assignment to [] position instead.
-     *              ie   str[newLength] = '\0';
-     */
-    _SQUID_INLINE_ void cut(size_t newLength);
-
-    /** \deprecated Use assignment to [] position instead.
-     *              ie   str[newLength] = '\0';
-     */
-    _SQUID_INLINE_ void cutPointer(char const *loc);
+    _SQUID_INLINE_ void cut(size_type newLength);
 
 #if DEBUGSTRINGS
-
     void stat (StoreEntry *) const;
-
 #endif
 
 
-
 private:
     void allocAndFill(const char *str, int len);
-    void allocBuffer(size_t sz);
-    void setBuffer(char *buf, size_t sz);
+    void allocBuffer(size_type sz);
+    void setBuffer(char *buf, size_type sz);
 
     /* never reference these directly! */
-    unsigned short int size_; /* buffer size; 64K limit */
+    size_type size_; /* buffer size; 64K limit */
 
-    unsigned short int len_;  /* current length  */
+    size_type len_;  /* current length  */
 
     char *buf_;
+
+    _SQUID_INLINE_ void set(char const *loc, char const ch);
+    _SQUID_INLINE_ void cutPointer(char const *loc);
+
 };
 
 _SQUID_INLINE_ std::ostream & operator<<(std::ostream& os, String const &aString);
