@@ -1,5 +1,5 @@
 /*
- * $Id: client_side_reply.cc,v 1.154 2008/02/16 17:42:27 rousskov Exp $
+ * $Id$
  *
  * DEBUG: section 88    Client-side Reply Routines
  * AUTHOR: Robert Collins (Originally Duane Wessels in client_side.c)
@@ -46,14 +46,14 @@
 #include "HttpRequest.h"
 #include "forward.h"
 #include "clientStream.h"
-#include "AuthUserRequest.h"
+#include "auth/UserRequest.h"
 #if USE_SQUID_ESI
-#include "ESI.h"
+#include "esi/Esi.h"
 #endif
 #include "MemObject.h"
 #include "fde.h"
-#include "ACLChecklist.h"
-#include "ACL.h"
+#include "acl/FilledChecklist.h"
+#include "acl/Gadgets.h"
 #if DELAY_POOLS
 #include "DelayPools.h"
 #endif
@@ -64,7 +64,7 @@ CBDATA_CLASS_INIT(clientReplyContext);
 
 /* Local functions */
 extern "C" CSS clientReplyStatus;
-extern ErrorState *clientBuildError(err_type, http_status, char const *, IPAddress &, HttpRequest *);
+extern ErrorState *clientBuildError(err_type, http_status, char const *, IpAddress &, HttpRequest *);
 
 /* privates */
 
@@ -93,7 +93,7 @@ clientReplyContext::clientReplyContext(ClientHttpRequest *clientContext) : http
 void
 clientReplyContext::setReplyToError(
     err_type err, http_status status, const HttpRequestMethod& method, char const *uri,
-    IPAddress &addr, HttpRequest * failedrequest, char *unparsedrequest,
+    IpAddress &addr, HttpRequest * failedrequest, const char *unparsedrequest,
     AuthUserRequest * auth_user_request)
 {
     ErrorState *errstate =
@@ -400,8 +400,8 @@ clientReplyContext::handleIMSReply(StoreIOBuffer result)
 extern "C" CSR clientGetMoreData;
 extern "C" CSD clientReplyDetach;
 
-/*
- * clientCacheHit should only be called until the HTTP reply headers
+/**
+ * clientReplyContext::cacheHit Should only be called until the HTTP reply headers
  * have been parsed.  Normally this should be a single call, but
  * it might take more than one.  As soon as we have the headers,
  * we hand off to clientSendMoreData, processExpired, or
@@ -411,12 +411,16 @@ void
 clientReplyContext::CacheHit(void *data, StoreIOBuffer result)
 {
     clientReplyContext *context = (clientReplyContext *)data;
-    context->cacheHit (result);
+    context->cacheHit(result);
 }
 
+/**
+ * Process a possible cache HIT.
+ */
 void
 clientReplyContext::cacheHit(StoreIOBuffer result)
 {
+    /** Ignore if the HIT object is being deleted. */
     if (deleting)
         return;
 
@@ -603,7 +607,7 @@ clientReplyContext::cacheHit(StoreIOBuffer result)
     }
 }
 
-/*
+/**
  * Prepare to fetch the object as it's a cache miss of some kind.
  */
 void
@@ -613,11 +617,11 @@ clientReplyContext::processMiss()
     HttpRequest *r = http->request;
     ErrorState *err = NULL;
     debugs(88, 4, "clientProcessMiss: '" << RequestMethodStr(r->method) << " " << url << "'");
-    /*
+
+    /**
      * We might have a left-over StoreEntry from a failed cache hit
      * or IMS request.
      */
-
     if (http->storeEntry()) {
         if (EBIT_TEST(http->storeEntry()->flags, ENTRY_SPECIAL)) {
             debugs(88, 0, "clientProcessMiss: miss on a special object (" << url << ").");
@@ -628,22 +632,24 @@ clientReplyContext::processMiss()
         removeClientStoreReference(&sc, http);
     }
 
+    /** Check if its a PURGE request to be actioned. */
     if (r->method == METHOD_PURGE) {
         purgeRequest();
         return;
     }
 
+    /** Check if its an 'OTHER' request. Purge all cached entries if so and continue. */
     if (r->method == METHOD_OTHER) {
-        // invalidate all cache entries
         purgeAllCached();
     }
 
+    /** Check if 'only-if-cached' flag is set. Action if so. */
     if (http->onlyIfCached()) {
         processOnlyIfCachedMiss();
         return;
     }
 
-    /*
+    /**
      * Deny loops when running in accelerator/transproxy mode.
      */
     if (http->flags.accel && r->flags.loopdetect) {
@@ -672,16 +678,18 @@ clientReplyContext::processMiss()
             return;
         }
 
+        /** Check for internal requests. Update Protocol info if so. */
         if (http->flags.internal)
             r->protocol = PROTO_INTERNAL;
 
+        /** Start forwarding to get the new object from network */
         FwdState::fwdStart(http->getConn() != NULL ? http->getConn()->fd : -1,
                            http->storeEntry(),
                            r);
     }
 }
 
-/*
+/**
  * client issued a request with an only-if-cached cache-control directive;
  * we did not find a cached object that can be returned without
  *     contacting other servers;
@@ -1283,7 +1291,7 @@ clientReplyContext::buildReplyHeader()
         int connection_auth_blocked = 0;
         while ((e = hdr->getEntry(&pos))) {
             if (e->id == HDR_WWW_AUTHENTICATE) {
-                const char *value = e->value.buf();
+                const char *value = e->value.rawBuf();
 
                 if ((strncasecmp(value, "NTLM", 4) == 0 &&
                         (value[4] == '\0' || value[4] == ' '))
@@ -1346,34 +1354,24 @@ clientReplyContext::buildReplyHeader()
 #endif
 
     /* Check whether we should send keep-alive */
-    // TODO: disable proxy_keepalive only once
-
-    if (reply->bodySize(request->method) < 0) {
-        debugs(88, 3, "clientBuildReplyHeader: can't keep-alive, unknown body size" );
-        request->flags.proxy_keepalive = 0;
-    }
-
-    if (fdUsageHigh()&& !request->flags.must_keepalive) {
-        debugs(88, 3, "clientBuildReplyHeader: Not many unused FDs, can't keep-alive");
-        request->flags.proxy_keepalive = 0;
-    }
-
     if (!Config.onoff.error_pconns && reply->sline.status >= 400 && !request->flags.must_keepalive) {
         debugs(33, 3, "clientBuildReplyHeader: Error, don't keep-alive");
         request->flags.proxy_keepalive = 0;
-    }
-
-    if (!Config.onoff.client_pconns && !request->flags.must_keepalive)
+    } else if (!Config.onoff.client_pconns && !request->flags.must_keepalive) {
+        debugs(33, 2, "clientBuildReplyHeader: Connection Keep-Alive not requested by admin or client");
         request->flags.proxy_keepalive = 0;
-
-    if (request->flags.proxy_keepalive && shutting_down) {
+    } else if (request->flags.proxy_keepalive && shutting_down) {
         debugs(88, 3, "clientBuildReplyHeader: Shutting down, don't keep-alive.");
         request->flags.proxy_keepalive = 0;
-    }
-
-    if (request->flags.connection_auth && !reply->keep_alive) {
+    } else if (request->flags.connection_auth && !reply->keep_alive) {
         debugs(33, 2, "clientBuildReplyHeader: Connection oriented auth but server side non-persistent");
         request->flags.proxy_keepalive = 0;
+    } else if (reply->bodySize(request->method) < 0) {
+        debugs(88, 3, "clientBuildReplyHeader: can't keep-alive, unknown body size" );
+        request->flags.proxy_keepalive = 0;
+    } else if (fdUsageHigh()&& !request->flags.must_keepalive) {
+        debugs(88, 3, "clientBuildReplyHeader: Not many unused FDs, can't keep-alive");
+        request->flags.proxy_keepalive = 0;
     }
 
 
@@ -1388,7 +1386,7 @@ clientReplyContext::buildReplyHeader()
                  ThisCache);
         strListAdd(&strVia, bbuf, ',');
         hdr->delById(HDR_VIA);
-        hdr->putStr(HDR_VIA, strVia.buf());
+        hdr->putStr(HDR_VIA, strVia.termedBuf());
     }
     /* Signal keep-alive if needed */
     hdr->putStr(http->flags.accel ? HDR_CONNECTION : HDR_PROXY_CONNECTION,
@@ -1439,12 +1437,17 @@ clientReplyContext::identifyStoreObject()
     }
 }
 
+/**
+ * Check state of the current StoreEntry object.
+ * to see if we can determine the final status of the request.
+ */
 void
 clientReplyContext::identifyFoundObject(StoreEntry *newEntry)
 {
     StoreEntry *e = newEntry;
     HttpRequest *r = http->request;
 
+    /** \li If the entry received isNull() then we ignore it. */
     if (e->isNull()) {
         http->storeEntry(NULL);
     } else {
@@ -1452,8 +1455,11 @@ clientReplyContext::identifyFoundObject(StoreEntry *newEntry)
     }
 
     e = http->storeEntry();
-    /* Release IP-cache entries on reload */
 
+    /* Release IP-cache entries on reload */
+    /** \li If the request has no-cache flag set or some no_cache HACK in operation we
+      * 'invalidate' the cached IP entries for this request ???
+      */
     if (r->flags.nocache) {
 
 #if USE_DNSSERVERS
@@ -1492,14 +1498,15 @@ clientReplyContext::identifyFoundObject(StoreEntry *newEntry)
 #endif
 
     if (NULL == http->storeEntry()) {
-        /* this object isn't in the cache */
-        debugs(85, 3, "clientProcessRequest2: storeGet() MISS");
+        /** \li If no StoreEntry object is current assume this object isn't in the cache set MISS*/
+        debugs(85, 3, "clientProcessRequest2: StoreEntry is NULL -  MISS");
         http->logType = LOG_TCP_MISS;
         doGetMoreData();
         return;
     }
 
     if (Config.onoff.offline) {
+        /** \li If we are running in offline mode set to HIT */
         debugs(85, 3, "clientProcessRequest2: offline HIT");
         http->logType = LOG_TCP_HIT;
         doGetMoreData();
@@ -1507,7 +1514,8 @@ clientReplyContext::identifyFoundObject(StoreEntry *newEntry)
     }
 
     if (http->redirect.status) {
-        /* force this to be a miss */
+        /** \li If redirection status is True force this to be a MISS */
+        debugs(85, 3, "clientProcessRequest2: redirectStatus forced StoreEntry to NULL -  MISS");
         http->storeEntry(NULL);
         http->logType = LOG_TCP_MISS;
         doGetMoreData();
@@ -1523,7 +1531,7 @@ clientReplyContext::identifyFoundObject(StoreEntry *newEntry)
     }
 
     if (EBIT_TEST(e->flags, ENTRY_SPECIAL)) {
-        /* Special entries are always hits, no matter what the client says */
+        /* \li Special entries are always hits, no matter what the client says */
         debugs(85, 3, "clientProcessRequest2: ENTRY_SPECIAL HIT");
         http->logType = LOG_TCP_HIT;
         doGetMoreData();
@@ -1543,13 +1551,14 @@ clientReplyContext::identifyFoundObject(StoreEntry *newEntry)
     doGetMoreData();
 }
 
-/* Request more data from the store for the client Stream
+/**
+ * Request more data from the store for the client Stream
  * This is *the* entry point to this module.
  *
  * Preconditions:
- * This is the head of the list.
- * There is at least one more node.
- * data context is not null
+ *  - This is the head of the list.
+ *  - There is at least one more node.
+ *  - Data context is not null
  */
 void
 clientGetMoreData(clientStreamNode * aNode, ClientHttpRequest * http)
@@ -1619,8 +1628,7 @@ clientReplyContext::doGetMoreData()
              * the method as GET.
              */
             http->storeEntry()->createMemObject(http->uri, http->log_uri);
-            http->storeEntry()->mem_obj->method =
-                http->request->method;
+            http->storeEntry()->mem_obj->method = http->request->method;
         }
 
         sc = storeClientListAdd(http->storeEntry(), this);
@@ -1635,9 +1643,9 @@ clientReplyContext::doGetMoreData()
         assert(http->out.size == 0);
         assert(http->out.offset == 0);
 #if USE_ZPH_QOS
-        if (Config.zph_tos_local) {
-            debugs(33, 2, "ZPH Local hit, TOS="<<Config.zph_tos_local);
-            comm_set_tos(http->getConn()->fd,Config.zph_tos_local);
+        if (Config.zph.tos_local_hit) {
+            debugs(33, 2, "ZPH Local hit, TOS=" << Config.zph.tos_local_hit);
+            comm_set_tos(http->getConn()->fd, Config.zph.tos_local_hit);
         }
 #endif /* USE_ZPH_QOS */
         tempBuffer.offset = reqofs;
@@ -1650,16 +1658,16 @@ clientReplyContext::doGetMoreData()
     }
 }
 
-/* the next node has removed itself from the stream. */
+/** The next node has removed itself from the stream. */
 void
 clientReplyDetach(clientStreamNode * node, ClientHttpRequest * http)
 {
-    /* detach from the stream */
+    /** detach from the stream */
     clientStreamDetach(node, http);
 }
 
-/*
- * accepts chunk of a http message in buf, parses prefix, filters headers and
+/**
+ * Accepts chunk of a http message in buf, parses prefix, filters headers and
  * such, writes processed message to the message recipient
  */
 void
@@ -1689,8 +1697,9 @@ clientReplyContext::errorInStream(StoreIOBuffer const &result, size_t const &siz
 void
 clientReplyContext::sendStreamError(StoreIOBuffer const &result)
 {
-    /* call clientWriteComplete so the client socket gets closed */
-    /* We call into the stream, because we don't know that there is a
+    /** call clientWriteComplete so the client socket gets closed
+     *
+     * We call into the stream, because we don't know that there is a
      * client socket!
      */
     debugs(88, 5, "clientReplyContext::sendStreamError: A stream error has occured, marking as complete and sending no data.");
@@ -1732,7 +1741,7 @@ clientReplyContext::next() const
 void
 clientReplyContext::sendBodyTooLargeError()
 {
-    IPAddress tmp_noaddr;
+    IpAddress tmp_noaddr;
     tmp_noaddr.SetNoAddr(); // TODO: make a global const
     ErrorState *err = clientBuildError(ERR_TOO_BIG, HTTP_FORBIDDEN, NULL,
                                        http->getConn() != NULL ? http->getConn()->peer : tmp_noaddr,
@@ -1746,8 +1755,10 @@ clientReplyContext::sendBodyTooLargeError()
 void
 clientReplyContext::processReplyAccess ()
 {
+    /* NP: this should probably soft-fail to a zero-sized-reply error ?? */
     assert(reply);
-    /* Dont't block our own responses or HTTP status messages */
+
+    /** Don't block our own responses or HTTP status messages */
     if (http->logType == LOG_TCP_DENIED ||
             http->logType == LOG_TCP_DENIED_REPLY ||
             alwaysAllowResponse(reply->sline.status)) {
@@ -1756,6 +1767,7 @@ clientReplyContext::processReplyAccess ()
         return;
     }
 
+    /** Check for reply to big error */
     if (reply->expectedBodyTooLarge(*http->request)) {
         sendBodyTooLargeError();
         return;
@@ -1763,13 +1775,15 @@ clientReplyContext::processReplyAccess ()
 
     headers_sz = reply->hdr_sz;
 
+    /** check for absent access controls (permit by default) */
     if (!Config.accessList.reply) {
         processReplyAccessResult(1);
         return;
     }
 
-    ACLChecklist *replyChecklist;
-    replyChecklist = clientAclChecklistCreate(Config.accessList.reply, http);
+    /** Process http_reply_access lists */
+    ACLFilledChecklist *replyChecklist =
+        clientAclChecklistCreate(Config.accessList.reply, http);
     replyChecklist->reply = HTTPMSGLOCK(reply);
     replyChecklist->nonBlockingCheck(ProcessReplyAccessResult, this);
 }
@@ -1800,7 +1814,7 @@ clientReplyContext::processReplyAccessResult(bool accessAllowed)
         if (page_id == ERR_NONE)
             page_id = ERR_ACCESS_DENIED;
 
-        IPAddress tmp_noaddr;
+        IpAddress tmp_noaddr;
         tmp_noaddr.SetNoAddr();
         err = clientBuildError(page_id, HTTP_FORBIDDEN, NULL,
                                http->getConn() != NULL ? http->getConn()->peer : tmp_noaddr,
@@ -1915,13 +1929,14 @@ clientReplyContext::sendMoreData (StoreIOBuffer result)
     if (reqofs==0 && !logTypeIsATcpHit(http->logType)) {
         assert(fd >= 0); // the beginning of this method implies fd may be -1
         int tos = 0;
-        if (Config.zph_tos_peer &&
-                (http->request->hier.code==SIBLING_HIT ||
-                 (Config.onoff.zph_tos_parent && http->request->hier.code==PARENT_HIT) ) ) {
-            tos = Config.zph_tos_peer;
-            debugs(33, 2, "ZPH: Peer hit with hier.code="<<http->request->hier.code<<", TOS="<<tos);
-        } else if (Config.onoff.zph_preserve_miss_tos && Config.zph_preserve_miss_tos_mask) {
-            tos = fd_table[fd].upstreamTOS & Config.zph_preserve_miss_tos_mask;
+        if (Config.zph.tos_sibling_hit && http->request->hier.code==SIBLING_HIT ) {
+            tos = Config.zph.tos_sibling_hit;
+            debugs(33, 2, "ZPH: Sibling Peer hit with hier.code=" << http->request->hier.code << ", TOS=" << tos);
+        } else if (Config.zph.tos_parent_hit && http->request->hier.code==PARENT_HIT) {
+            tos = Config.zph.tos_parent_hit;
+            debugs(33, 2, "ZPH: Parent Peer hit with hier.code=" << http->request->hier.code << ", TOS=" << tos);
+        } else if (Config.zph.preserve_miss_tos && Config.zph.preserve_miss_tos_mask) {
+            tos = fd_table[fd].upstreamTOS & Config.zph.preserve_miss_tos_mask;
             debugs(33, 2, "ZPH: Preserving TOS on miss, TOS="<<tos);
         }
         comm_set_tos(fd,tos);
@@ -2033,7 +2048,7 @@ clientReplyContext::createStoreEntry(const HttpRequestMethod& m, request_flags f
 
 ErrorState *
 clientBuildError(err_type page_id, http_status status, char const *url,
-                 IPAddress &src_addr, HttpRequest * request)
+                 IpAddress &src_addr, HttpRequest * request)
 {
     ErrorState *err = errorCon(page_id, status, request);
     err->src_addr = src_addr;
