@@ -33,8 +33,8 @@
 
 #include "squid.h"
 #include "forward.h"
-#include "ACLChecklist.h"
-#include "ACL.h"
+#include "acl/FilledChecklist.h"
+#include "acl/Gadgets.h"
 #include "CacheManager.h"
 #include "event.h"
 #include "errorpage.h"
@@ -201,16 +201,13 @@ FwdState::fwdStart(int client_fd, StoreEntry *entry, HttpRequest *request)
      */
 
     if ( Config.accessList.miss && !request->client_addr.IsNoAddr() &&
-         request->protocol != PROTO_INTERNAL && request->protocol != PROTO_CACHEOBJ) {
+            request->protocol != PROTO_INTERNAL && request->protocol != PROTO_CACHEOBJ) {
         /**
          * Check if this host is allowed to fetch MISSES from us (miss_access)
          */
-        ACLChecklist ch;
+        ACLFilledChecklist ch(Config.accessList.miss, request, NULL);
         ch.src_addr = request->client_addr;
         ch.my_addr = request->my_addr;
-        ch.request = HTTPMSGLOCK(request);
-        ch.accessList = cbdataReference(Config.accessList.miss);
-        /* cbdataReferenceDone() happens in either fastCheck() or ~ACLCheckList */
         int answer = ch.fastCheck();
 
         if (answer == 0) {
@@ -664,7 +661,7 @@ FwdState::initiateSSL()
     // Create the ACL check list now, while we have access to more info.
     // The list is used in ssl_verify_cb() and is freed in ssl_free().
     if (acl_access *acl = Config.ssl_client.cert_error) {
-        ACLChecklist *check = aclChecklistCreate(acl, request, dash_str);
+        ACLFilledChecklist *check = new ACLFilledChecklist(acl, request, dash_str);
         check->fd(fd);
         SSL_set_ex_data(ssl, ssl_ex_index_cert_error_check, check);
     }
@@ -826,12 +823,11 @@ FwdState::connectStart()
         return;
     }
 
-    if(fs->_peer) {
+    if (fs->_peer) {
         host = fs->_peer->host;
         port = fs->_peer->http_port;
         fd = fwdPconnPool->pop(fs->_peer->name, fs->_peer->http_port, request->GetHost(), client_addr, checkRetriable());
-    }
-    else {
+    } else {
         host = request->GetHost();
         port = request->port;
         fd = fwdPconnPool->pop(host, port, NULL, client_addr, checkRetriable());
@@ -1182,10 +1178,10 @@ FwdState::reforwardableStatus(http_status s)
 /**
  * Decide where details need to be gathered to correctly describe a persistent connection.
  * What is needed:
- * \item  host name of server at other end of this link (either peer or requested host)
- * \item  port to which we connected the other end of this link (for peer or request)
- * \item  domain for which the connection is supposed to be used
- * \item  address of the client for which we made the connection
+ *  -  host name of server at other end of this link (either peer or requested host)
+ *  -  port to which we connected the other end of this link (for peer or request)
+ *  -  domain for which the connection is supposed to be used
+ *  -  address of the client for which we made the connection
  */
 void
 FwdState::pconnPush(int fd, const peer *_peer, const HttpRequest *req, const char *domain, IpAddress &client_addr)
@@ -1341,8 +1337,6 @@ aclMapTOS(acl_tos * head, ACLChecklist * ch)
 IpAddress
 getOutgoingAddr(HttpRequest * request, struct peer *dst_peer)
 {
-    ACLChecklist ch;
-
     if (request && request->flags.spoof_client_ip)
         return request->client_addr;
 
@@ -1350,12 +1344,12 @@ getOutgoingAddr(HttpRequest * request, struct peer *dst_peer)
         return IpAddress(); // anything will do.
     }
 
+    ACLFilledChecklist ch(NULL, request, NULL);
     ch.dst_peer = dst_peer;
 
     if (request) {
         ch.src_addr = request->client_addr;
         ch.my_addr = request->my_addr;
-        ch.request = HTTPMSGLOCK(request);
     }
 
     return aclMapAddr(Config.accessList.outgoing_address, &ch);
@@ -1364,12 +1358,11 @@ getOutgoingAddr(HttpRequest * request, struct peer *dst_peer)
 unsigned long
 getOutgoingTOS(HttpRequest * request)
 {
-    ACLChecklist ch;
+    ACLFilledChecklist ch(NULL, request, NULL);
 
     if (request) {
         ch.src_addr = request->client_addr;
         ch.my_addr = request->my_addr;
-        ch.request = HTTPMSGLOCK(request);
     }
 
     return aclMapTOS(Config.accessList.outgoing_tos, &ch);
