@@ -1,5 +1,5 @@
 /*
- * $Id: fqdncache.cc,v 1.178 2008/02/26 21:49:34 amosjeffries Exp $
+ * $Id$
  *
  * DEBUG: section 35    FQDN Cache
  * AUTHOR: Harvest Derived
@@ -516,7 +516,7 @@ fqdncacheHandleReply(void *data, rfc1035_rr * answers, int na, const char *error
  * 			and does not affect the FQDN cache.
  */
 void
-fqdncache_nbgethostbyaddr(IPAddress &addr, FQDNH * handler, void *handlerData)
+fqdncache_nbgethostbyaddr(IpAddress &addr, FQDNH * handler, void *handlerData)
 {
     fqdncache_entry *f = NULL;
     char name[MAX_IPSTRLEN];
@@ -634,12 +634,15 @@ fqdncache_init(void)
  *
  */
 const char *
-fqdncache_gethostbyaddr(IPAddress &addr, int flags)
+fqdncache_gethostbyaddr(IpAddress &addr, int flags)
 {
     char name[MAX_IPSTRLEN];
     fqdncache_entry *f = NULL;
 
-    assert(!addr.IsAnyAddr() && !addr.IsNoAddr());
+    if (addr.IsAnyAddr() || addr.IsNoAddr()) {
+        return NULL;
+    }
+
     addr.NtoA(name,MAX_IPSTRLEN);
     FqdncacheStats.requests++;
     f = fqdncache_get(name);
@@ -736,7 +739,7 @@ dummy_handler(const char *bufnotused, void *datanotused)
 
 /// \ingroup FQDNCacheAPI
 const char *
-fqdnFromAddr(IPAddress &addr)
+fqdnFromAddr(IpAddress &addr)
 {
     const char *n;
     static char buf[MAX_IPSTRLEN];
